'============
'TANKDRUM.BAS v1.0d
'============
'A Playable/Recordable Tankdrum Virtual Instrument
'Coded by Dav for QB64 Phoenix Edition, MAY/2022

'New in v1.0d:
'               - Sounding notes are now highlighted.
'               - Added Menu/Help screen.  Press M for it.
'               - Added file saving box.
'               - Added loading a file box.
'               - changed song extensions to .TNK
'               - Included several sample .TNK files to load.
'               - made some small bug fixes.

'-----
'ABOUT
'-----

'QB64 TankDrum is a virtual instrument that you can play
'and record little tunes with.  Use the mouse to click on
'and play notes. The program starts in freeplay mode, but
'you can record in real time as you play by pressing R.
'Record over your song several times to add more notes and
'make more complex patterns (called overdubbing).

'--------
'CONTROLS
'--------

'Press R to record song and to overdub (add) notes to a song.
'Press U to Undo last recording (in case you nmake a mistake!
'Press P to play back the recorded song currently in memory.
'Press C to clear (erase) the recorded song currently in memory.

'Press ESC to stop recording and to stop playing back of the song.
'NOTE: You can also play notes during a song play back.

'Press S to Save your recording in the current dir.
'Press L to Load a .TNK recording file (if any exists).
'Press D to Delete a currently loaded .TNK file.


$EXEICON:'tankdrum\img\tankdrum.ico'
_ICON

DIM SHARED aflag, atimer, bflag, btimer, cflag, ctimer, dflag, dtimer
DIM SHARED eflag, etimer, fflag, ftimer, gflag, gtimer, hflag, htimer
DIM SHARED iflag, itimer, jflag, jtimer, kflag, ktimer, lflag, ltimer
DIM SHARED mflag, mtimer, nflag, ntimer, oflag, otimer

DIM SHARED a&, b&, c&, d&, e&, f&, g&, h&, i&, j&, k&, l&, m&, n&, o&

SCREEN _LOADIMAGE("img\tankdrum.png", 32)

DO: LOOP UNTIL _SCREENEXISTS 'Make sure window exists before TITLE used.
_TITLE "QB64 TankDrum v1.0d"

'Load sound samples
a& = _SNDOPEN("ogg\a.ogg")
b& = _SNDOPEN("ogg\b.ogg")
c& = _SNDOPEN("ogg\c.ogg")
d& = _SNDOPEN("ogg\d.ogg")
e& = _SNDOPEN("ogg\e.ogg")
f& = _SNDOPEN("ogg\f.ogg")
g& = _SNDOPEN("ogg\g.ogg")
h& = _SNDOPEN("ogg\h.ogg")
i& = _SNDOPEN("ogg\i.ogg")
j& = _SNDOPEN("ogg\j.ogg")
k& = _SNDOPEN("ogg\k.ogg")
l& = _SNDOPEN("ogg\l.ogg")
m& = _SNDOPEN("ogg\m.ogg")
n& = _SNDOPEN("ogg\n.ogg")
o& = _SNDOPEN("ogg\o.ogg")

MaxSongLength = 500000 'max of 500k bytes song data

'play a welcome scale (all notes)
FOR d = 97 TO 111
    PlayNote CHR$(d)
    GOSUB NoteDisplay
    _DELAY .05
NEXT

'========
MainLoop:
'========

GOSUB DrawTopBar

DO

    GOSUB PlayDrum

    k$ = UCASE$(INKEY$)
    IF k$ = "R" THEN GOSUB record: GOTO MainLoop
    IF k$ = "P" THEN GOSUB playback: GOTO MainLoop
    IF k$ = "C" THEN
        NoteData$ = ""
        loadedfile$ = ""
        ok$ = IBOX$("SONG MEMORY CLEARED!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
        GOTO MainLoop
    END IF
    IF k$ = "U" THEN
        IF NoteData$ <> "" THEN
            NoteData$ = UndoData$
            ok$ = IBOX$("LAST RECORDING UNDONE!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
        ELSE
            ok$ = IBOX$("NO RECORDING TO UNDO!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
        END IF
        GOTO MainLoop
    END IF
    IF k$ = "S" THEN
        IF NoteData$ <> "" THEN
            fname$ = IBOX$("SAVE FILE AS: ", 20, _RGB(255, 255, 255), _RGB(70, 95, 114), 0)
            IF fname$ <> "" THEN
                IF INSTR(1, UCASE$(fname$), ".TNK") = 0 THEN fname$ = fname$ + ".tnk"
                OPEN fname$ FOR OUTPUT AS #1
                PRINT #1, NoteData$;: CLOSE 1
                ok$ = IBOX$(UCASE$(fname$) + " HAS BEEN SAVED!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
                loadedfile$ = fname$
            END IF
        ELSE
            ok$ = IBOX$("THERE IS NO SONG TO SAVE!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
        END IF
        GOTO MainLoop
    END IF ' _RGB(70, 95, 114)
    IF k$ = "L" THEN
        lf$ = FileSelect$(5, 15, 20, 45, "*.tnk", _RGB(32, 32, 64), _RGB(60, 85, 104), _RGB(255, 255, 255), _RGB(255, 255, 255), _RGB(232, 232, 0))
        IF lf$ <> "" THEN
            loadedfile$ = MID$(lf$, _INSTRREV(lf$, "\") + 1)
            OPEN loadedfile$ FOR BINARY AS #1
            NoteData$ = INPUT$(LOF(1), 1): CLOSE 1
            plf$ = UCASE$(loadedfile$) + " LOADED!"
            ok$ = IBOX$(plf$, 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
        ELSE
            ok$ = IBOX$("NO SONG FILE SELECTED!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
        END IF

        GOTO MainLoop

    END IF

    IF k$ = "D" THEN
        IF _FILEEXISTS(loadedfile$) THEN
            ''=== ask make sure here
            ok$ = IBOX$("DELETE " + UCASE$(loadedfile$) + " (Y/N)? ", 1, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
            huh$ = UCASE$(ok$)
            IF huh$ = "Y" THEN
                KILL loadedfile$
                ok$ = IBOX$(UCASE$(loadedfile$) + " DELETED!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
                loadedfile$ = ""
            END IF
        ELSE
            IF loadedfile$ = "" THEN
                ok$ = IBOX$(" NO SONG FILE IS LOADED!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
            ELSE
                ok$ = IBOX$(UCASE$(loadedfile$) + " NOT FOUND!", 0, _RGB(255, 255, 255), _RGB(70, 95, 114), 2)
            END IF
        END IF

        GOTO MainLoop

    END IF

    IF k$ = "M" THEN
        backup& = _COPYIMAGE(_DISPLAY)
        temp& = _LOADIMAGE("img\menu.png", 32)
        _PUTIMAGE (120, 120), temp&
        hmm$ = INPUT$(1)
        _PUTIMAGE (0, 0), backup&
        _FREEIMAGE temp&
        _FREEIMAGE backup&
    END IF

LOOP

END

'=========
DrawTopBar:
'=========
LINE (0, 0)-(_WIDTH, 16), _RGB(32, 32, 64), BF
COLOR _RGB(255, 255, 255), _RGB(32, 32, 64): LOCATE 1, 1
IF loadedfile$ = "" AND NoteData$ = "" THEN PRINT " Song: (none)";
IF loadedfile$ = "" AND NoteData$ <> "" THEN PRINT " Song: (unsaved)";
IF loadedfile$ <> "" THEN PRINT " Song: "; loadedfile$
LOCATE 1, 63: PRINT "M = MENU";
RETURN

'=======
record:
'=======

'Make copy of NoteData$ for undo purposes
UndoData$ = NoteData$

IF NoteData$ = "" THEN NoteData$ = "1" 'song recording marker
note = 1

COLOR _RGB(255, 255, 255), _RGB(255, 0, 0)
LOCATE 1, 1: PRINT " Now Recording | ESC to Stop           ";

DO

    note$ = ""

    'playback previously recorded notes
    SELECT CASE MID$(NoteData$, note, 1)
        CASE "a": PlayNote "a"
        CASE "b": PlayNote "b"
        CASE "c": PlayNote "c"
        CASE "d": PlayNote "d"
        CASE "e": PlayNote "e"
        CASE "f": PlayNote "f"
        CASE "g": PlayNote "g"
        CASE "h": PlayNote "h"
        CASE "i": PlayNote "i"
        CASE "j": PlayNote "j"
        CASE "k": PlayNote "k"
        CASE "l": PlayNote "l"
        CASE "m": PlayNote "m"
        CASE "n": PlayNote "n"
        CASE "o": PlayNote "o"
    END SELECT

    'Add these notes to recording...
    mi = _MOUSEINPUT
    IF _MOUSEBUTTON(1) = 0 THEN done = 0
    IF _MOUSEBUTTON(1) AND done = 0 THEN
        mx = _MOUSEX: my = _MOUSEY
        IF mx > 217 AND mx < 269 AND my > 226 AND my < 355 THEN PlayNote "a": note$ = "a"
        IF mx > 276 AND mx < 329 AND my > 226 AND my < 304 THEN PlayNote "b": note$ = "b"
        IF mx > 335 AND mx < 382 AND my > 226 AND my < 355 THEN PlayNote "c": note$ = "c"
        IF mx > 271 AND mx < 328 AND my > 402 AND my < 545 THEN PlayNote "d": note$ = "d"
        IF mx > 351 AND mx < 432 AND my > 407 AND my < 510 THEN PlayNote "e": note$ = "e"
        IF mx > 414 AND mx < 507 AND my > 346 AND my < 424 THEN PlayNote "f": note$ = "f"
        IF mx > 412 AND mx < 523 AND my > 265 AND my < 321 THEN PlayNote "g": note$ = "g"
        IF mx > 416 AND mx < 493 AND my > 175 AND my < 242 THEN PlayNote "h": note$ = "h"
        IF mx > 354 AND mx < 420 AND my > 111 AND my < 182 THEN PlayNote "i": note$ = "i"
        IF mx > 266 AND mx < 334 AND my > 70 AND my < 184 THEN PlayNote "j": note$ = "j"
        IF mx > 179 AND mx < 244 AND my > 113 AND my < 182 THEN PlayNote "k": note$ = "k"
        IF mx > 109 AND mx < 182 AND my > 174 AND my < 240 THEN PlayNote "l": note$ = "l"
        IF mx > 83 AND mx < 184 AND my > 265 AND my < 320 THEN PlayNote "m": note$ = "m"
        IF mx > 98 AND mx < 185 AND my > 345 AND my < 414 THEN PlayNote "n": note$ = "n"
        IF mx > 173 AND mx < 245 AND my > 405 AND my < 508 THEN PlayNote "o": note$ = "o"
        done = 1 'did a mouse click
    END IF

    GOSUB NoteDisplay

    IF note$ <> "" THEN
        IF note <= LEN(NoteData$) THEN
            MID$(NoteData$, note, 1) = note$ 'add the note
        ELSE
            NoteData$ = NoteData$ + note$
        END IF
    ELSE
        IF note >= LEN(NoteData$) THEN
            NoteData$ = NoteData$ + " " 'else add empty space
        END IF
    END IF

    note = note + 1
    IF note >= MaxSongLength THEN EXIT DO

    _LIMIT 1500

LOOP UNTIL INKEY$ = CHR$(27)

DO: LOOP UNTIL INKEY$ = ""

RETURN


'========
playback:
'========

'If nothing recorded yet, don't try play
IF NoteData$ = "" THEN
    COLOR _RGB(0, 0, 0), _RGB(255, 255, 0)
    LOCATE 1, 1: PRINT " No recorded song in memory! ";
    _DELAY 2
    DO: LOOP UNTIL INKEY$ = ""
    RETURN
END IF

COLOR _RGB(0, 0, 0), _RGB(0, 255, 0)
LOCATE 1, 1: PRINT " Playing song | ESC to stop            ";

note = 1

DO
    SELECT CASE MID$(NoteData$, note, 1)
        CASE "a": PlayNote "a"
        CASE "b": PlayNote "b"
        CASE "c": PlayNote "c"
        CASE "d": PlayNote "d"
        CASE "e": PlayNote "e"
        CASE "f": PlayNote "f"
        CASE "g": PlayNote "g"
        CASE "h": PlayNote "h"
        CASE "i": PlayNote "i"
        CASE "j": PlayNote "j"
        CASE "k": PlayNote "k"
        CASE "l": PlayNote "l"
        CASE "m": PlayNote "m"
        CASE "n": PlayNote "n"
        CASE "o": PlayNote "o"
    END SELECT

    note = note + 1

    IF note >= LEN(NoteData$) THEN EXIT DO

    _LIMIT 1500

    GOSUB PlayDrum 'you can play notes too while song is playing

LOOP UNTIL INKEY$ = CHR$(27)

DO: LOOP UNTIL INKEY$ = ""

RETURN


'========
PlayDrum: 'Gets mouse input and plays drum notes
'========

mi = _MOUSEINPUT
IF _MOUSEBUTTON(1) = 0 THEN done = 0

IF _MOUSEBUTTON(1) AND done = 0 THEN
    mx = _MOUSEX: my = _MOUSEY
    IF mx > 217 AND mx < 269 AND my > 226 AND my < 355 THEN PlayNote "a"
    IF mx > 276 AND mx < 329 AND my > 226 AND my < 304 THEN PlayNote "b"
    IF mx > 335 AND mx < 382 AND my > 226 AND my < 355 THEN PlayNote "c"
    IF mx > 271 AND mx < 328 AND my > 402 AND my < 545 THEN PlayNote "d"
    IF mx > 351 AND mx < 432 AND my > 407 AND my < 510 THEN PlayNote "e"
    IF mx > 414 AND mx < 507 AND my > 346 AND my < 424 THEN PlayNote "f"
    IF mx > 412 AND mx < 523 AND my > 265 AND my < 321 THEN PlayNote "g"
    IF mx > 416 AND mx < 493 AND my > 175 AND my < 242 THEN PlayNote "h"
    IF mx > 354 AND mx < 420 AND my > 111 AND my < 182 THEN PlayNote "i"
    IF mx > 266 AND mx < 334 AND my > 70 AND my < 184 THEN PlayNote "j"
    IF mx > 179 AND mx < 244 AND my > 113 AND my < 182 THEN PlayNote "k"
    IF mx > 109 AND mx < 182 AND my > 174 AND my < 240 THEN PlayNote "l"
    IF mx > 83 AND mx < 184 AND my > 265 AND my < 320 THEN PlayNote "m"
    IF mx > 98 AND mx < 185 AND my > 345 AND my < 414 THEN PlayNote "n"
    IF mx > 173 AND mx < 245 AND my > 405 AND my < 508 THEN PlayNote "o"
    done = 1
END IF

GOSUB NoteDisplay

RETURN

'===========
NoteDisplay:
'===========

IF aflag = 1 THEN
    IF TIMER - atimer > .5 THEN
        aflag = 0: NotePut "img\a0.png", 215, 225
    END IF
END IF
IF bflag = 1 THEN
    IF TIMER - btimer > .5 THEN
        bflag = 0: NotePut "img\b0.png", 275, 225
    END IF
END IF
IF cflag = 1 THEN
    IF TIMER - ctimer > .5 THEN
        cflag = 0: NotePut "img\c0.png", 335, 225
    END IF
END IF
IF dflag = 1 THEN
    IF TIMER - dtimer > .5 THEN
        dflag = 0: NotePut "img\d0.png", 271, 402
    END IF
END IF
IF eflag = 1 THEN
    IF TIMER - etimer > .5 THEN
        eflag = 0: NotePut "img\e0.png", 351, 402
    END IF
END IF
IF fflag = 1 THEN
    IF TIMER - ftimer > .5 THEN
        fflag = 0: NotePut "img\f0.png", 415, 338
    END IF
END IF
IF gflag = 1 THEN
    IF TIMER - gtimer > .5 THEN
        gflag = 0: NotePut "img\g0.png", 414, 265
    END IF
END IF
IF hflag = 1 THEN
    IF TIMER - htimer > .5 THEN
        hflag = 0: NotePut "img\h0.png", 416, 172
    END IF
END IF
IF iflag = 1 THEN
    IF TIMER - itimer > .5 THEN
        iflag = 0: NotePut "img\i0.png", 354, 118
    END IF
END IF
IF jflag = 1 THEN
    IF TIMER - jtimer > .5 THEN
        jflag = 0: NotePut "img\j0.png", 267, 94
    END IF
END IF
IF kflag = 1 THEN
    IF TIMER - ktimer > .5 THEN
        kflag = 0: NotePut "img\k0.png", 184, 115
    END IF
END IF
IF lflag = 1 THEN
    IF TIMER - ltimer > .5 THEN
        lflag = 0: NotePut "img\l0.png", 119, 170
    END IF
END IF
IF mflag = 1 THEN
    IF TIMER - mtimer > .5 THEN
        mflag = 0: NotePut "img\m0.png", 109, 266
    END IF
END IF
IF nflag = 1 THEN
    IF TIMER - ntimer > .5 THEN
        nflag = 0: NotePut "img\n0.png", 111, 333
    END IF
END IF
IF oflag = 1 THEN
    IF TIMER - otimer > .5 THEN
        oflag = 0: NotePut "img\o0.png", 173, 404
    END IF
END IF

RETURN

SUB NotePut (file$, x&, y&)
    temp& = _LOADIMAGE(file$, 32)
    _PUTIMAGE (x&, y&), temp&
    _FREEIMAGE temp&
END SUB

SUB PlayNote (note$)
    'Plays note$, draws note$, sets flags and timers on
    IF note$ = "a" THEN
        _SNDPLAYCOPY a&: aflag = 1: atimer = TIMER
        NotePut "img\a1.png", 215, 225
    END IF
    IF note$ = "b" THEN
        _SNDPLAYCOPY b&: bflag = 1: btimer = TIMER
        NotePut "img\b1.png", 275, 225
    END IF
    IF note$ = "c" THEN
        _SNDPLAYCOPY c&: cflag = 1: ctimer = TIMER
        NotePut "img\c1.png", 335, 225
    END IF
    IF note$ = "d" THEN
        _SNDPLAYCOPY d&: dflag = 1: dtimer = TIMER
        NotePut "img\d1.png", 271, 402
    END IF
    IF note$ = "e" THEN
        _SNDPLAYCOPY e&: eflag = 1: etimer = TIMER
        NotePut "img\e1.png", 351, 402
    END IF
    IF note$ = "f" THEN
        _SNDPLAYCOPY f&: fflag = 1: ftimer = TIMER
        NotePut "img\f1.png", 415, 338
    END IF
    IF note$ = "g" THEN
        _SNDPLAYCOPY g&: gflag = 1: gtimer = TIMER
        NotePut "img\g1.png", 414, 265
    END IF
    IF note$ = "h" THEN
        _SNDPLAYCOPY h&: hflag = 1: htimer = TIMER
        NotePut "img\h1.png", 416, 172
    END IF
    IF note$ = "i" THEN
        _SNDPLAYCOPY i&: iflag = 1: itimer = TIMER
        NotePut "img\i1.png", 354, 118
    END IF
    IF note$ = "j" THEN
        _SNDPLAYCOPY j&: jflag = 1: jtimer = TIMER
        NotePut "img\j1.png", 267, 94
    END IF
    IF note$ = "k" THEN
        _SNDPLAYCOPY k&: kflag = 1: ktimer = TIMER
        NotePut "img\k1.png", 184, 115
    END IF
    IF note$ = "l" THEN
        _SNDPLAYCOPY l&: lflag = 1: ltimer = TIMER
        NotePut "img\l1.png", 119, 170
    END IF
    IF note$ = "m" THEN
        _SNDPLAYCOPY m&: mflag = 1: mtimer = TIMER
        NotePut "img\m1.png", 109, 266
    END IF
    IF note$ = "n" THEN
        _SNDPLAYCOPY n&: nflag = 1: ntimer = TIMER
        NotePut "img\n1.png", 111, 333
    END IF
    IF note$ = "o" THEN
        _SNDPLAYCOPY o&: oflag = 1: otimer = TIMER
        NotePut "img\o1.png", 173, 404
    END IF
END SUB


FUNCTION FileSelect$ (y, x, y2, x2, Filespec$, fsborder&, fsback&, fsfile&, fsdir&, fshigh&)

    'EDITED VERSION OF FILESELECT$ ADAPTED FOR THIS PROGRAM

    '=== save original place of cursor
    origy = CSRLIN
    origx = POS(1)

    '=== save colors
    fg& = _DEFAULTCOLOR
    bg& = _BACKGROUNDCOLOR

    '=== Save whole screen
    DIM scr1 AS _MEM, scr2 AS _MEM
    scr1 = _MEMIMAGE(0): scr2 = _MEMNEW(scr1.SIZE)
    _MEMCOPY scr1, scr1.OFFSET, scr1.SIZE TO scr2, scr2.OFFSET

    '=== Generate a unique temp filename to use based on date + timer
    tmp$ = "_qb64_" + DATE$ + "_" + LTRIM$(STR$(INT(TIMER))) + ".tmp"
    IF INSTR(_OS$, "LINUX") THEN tmp$ = "/tmp/" + tmp$

    loadagain:

    top = 0
    selection = 0

    '=== make room for names
    REDIM FileNames$(10000) 'space for 10000 filenames

    '=== now grab list of files...
    IF INSTR(_OS$, "LINUX") THEN
        IF Filespec$ = "*.*" THEN Filespec$ = ""
        SHELL _HIDE "find -maxdepth 1 -type f -name '" + Filespec$ + "*' > " + tmp$
    ELSE
        SHELL _HIDE "dir /b /A:-D " + Filespec$ + " > " + tmp$
    END IF

    '=== open temp file
    FF = FREEFILE
    OPEN tmp$ FOR INPUT AS #FF

    WHILE ((LineCount < UBOUND(FileNames$)) AND (NOT EOF(FF)))

        LINE INPUT #FF, rl$

        '=== load, ignoring the generated temp file...
        IF rl$ <> tmp$ THEN

            'also remove the ./ added at the beginning when under linux
            IF INSTR(_OS$, "LINUX") THEN
                IF LEFT$(rl$, 2) = "./" THEN
                    rl$ = RIGHT$(rl$, LEN(rl$) - 2)
                END IF
            END IF

            FileNames$(LineCount) = rl$
            LineCount = LineCount + 1
        END IF

    WEND
    CLOSE #FF

    '=== Remove the temp file created
    IF INSTR(_OS$, "LINUX") THEN
        SHELL _HIDE "rm " + tmp$
    ELSE
        SHELL _HIDE "del " + tmp$
    END IF


    '=== draw a box
    COLOR fsborder&
    FOR l = 0 TO y2 + 1
        LOCATE y + l, x: PRINT STRING$(x2 + 4, CHR$(219));
    NEXT

    '=== show notice at top
    COLOR _RGB(255, 255, 128), fsborder&
    CurDir$ = _CWD$
    CurDir$ = MID$(CurDir$, 1, x2 - x - 3) + "..."

    LOCATE y, x + 16: PRINT "SELECT A SONG";

    '=== scroll through list...
    DO

        FOR l = 0 TO (y2 - 1)

            LOCATE (y + 1) + l, (x + 2)
            IF l + top = selection THEN
                COLOR fsback&, fshigh& 'selected line
            ELSE
                COLOR fsfile&, fsback& 'regular
                '=== directories get a different color...
                IF MID$(FileNames$(top + l), 1, 1) = "[" THEN
                    COLOR fsdir&, fsback&
                END IF
            END IF

            PRINT LEFT$(FileNames$(top + l) + STRING$(x2, " "), x2);

        NEXT

        '=== Get user input

        k$ = INKEY$
        SELECT CASE k$

            CASE IS = CHR$(0) + CHR$(72) 'Up arrow
                IF selection > 0 THEN selection = selection - 1
                IF selection < top THEN top = selection

            CASE IS = CHR$(0) + CHR$(80) 'Down Arrow
                IF selection < (LineCount - 1) THEN selection = selection + 1
                IF selection > (top + (y2 - 2)) THEN top = selection - y2 + 1

            CASE IS = CHR$(0) + CHR$(73) 'Page up
                top = top - y2
                selection = selection - y2
                IF top < 0 THEN top = 0
                IF selection < 0 THEN selection = 0

            CASE IS = CHR$(0) + CHR$(81) 'Page Down
                top = top + y2
                selection = selection + y2
                IF top >= LineCount - y2 THEN top = LineCount - y2
                IF top < 0 THEN top = 0
                IF selection >= LineCount THEN selection = LineCount - 1

            CASE IS = CHR$(0) + CHR$(71) 'Home
                top = 0: selection = 0

            CASE IS = CHR$(0) + CHR$(79) 'End
                selection = LineCount - 1
                top = selection - y2 + 1
                IF top < 0 THEN top = 0

            CASE IS = CHR$(27) ' ESC cancels
                FileSelect$ = ""
                EXIT DO

            CASE IS = CHR$(13) 'Enter

                IF RTRIM$(FileNames$(selection)) = "" THEN
                    FileSelect$ = "": EXIT DO
                END IF

                IF INSTR(_OS$, "LINUX") THEN
                    IF RIGHT$(_CWD$, 1) = "/" THEN
                        C$ = _CWD$
                    ELSE
                        C$ = _CWD$ + "/"
                    END IF
                ELSE
                    IF RIGHT$(_CWD$, 1) = "\" THEN
                        C$ = _CWD$
                    ELSE
                        C$ = _CWD$ + "\"
                    END IF
                END IF

                FileSelect$ = C$ + RTRIM$(FileNames$(selection))
                EXIT DO

        END SELECT

    LOOP

    _KEYCLEAR

    '=== Restore the whole screen
    _MEMCOPY scr2, scr2.OFFSET, scr2.SIZE TO scr1, scr1.OFFSET
    _MEMFREE scr1: _MEMFREE scr2

    '=== restore original y,x and color
    LOCATE origy, origx

    COLOR fg&, bg&

END FUNCTION


SUB DrawBox 'Draws the litte info box.  Used often
    LINE (150, 200)-(450, 270), _RGB(70, 95, 114), BF
    LINE (152, 202)-(448, 268), _RGB(255, 255, 255), B
    COLOR _RGB(255, 255, 255), _RGB(70, 95, 114)
END SUB

FUNCTION KINPUT$ (y, x, text$, limitnum)

    LOCATE y, x: PRINT text$;

    entry$ = ""
    y = CSRLIN: x = POS(1)

    DO
        a$ = INPUT$(1)

        IF a$ = CHR$(13) THEN 'enter returns entry
            KINPUT$ = entry$: EXIT FUNCTION
        END IF

        IF a$ = CHR$(27) THEN 'ESC bails out
            KINPUT$ = "": EXIT FUNCTION
        END IF

        IF a$ = CHR$(8) THEN 'Backspace goes back a space
            IF LEN(entry$) > 0 THEN
                entry$ = MID$(entry$, 1, LEN(entry$) - 1)
            END IF
        ELSE
            'add letter entered, if not over limitnum
            IF LEN(entry$) < limitnum THEN
                entry$ = entry$ + a$
            END IF
        END IF

        LOCATE y, x: PRINT SPACE$(limitnum);
        LOCATE y, x: PRINT entry$;

    LOOP

END FUNCTION


FUNCTION IBOX$ (text$, limitnum, fg&, bg&, delay)
    'This function either gets user input or shows info.
    'If limitnum is 0, it just shows info.

    'text$: Your text to show
    'limitnum: how many letters of input to get
    'fg&: The Text color
    'bg&: Background color of box
    'delay: optional delay period for message box only
    '       If set at -1 then wait for a keypress

    '=== save original place of cursor
    origy = CSRLIN: origx = POS(1)

    '=== Save whole screen
    DIM scr1 AS _MEM, scr2 AS _MEM
    scr1 = _MEMIMAGE(0): scr2 = _MEMNEW(scr1.SIZE)
    _MEMCOPY scr1, scr1.OFFSET, scr1.SIZE TO scr2, scr2.OFFSET

    '=== find center x/y of screen
    y = INT(_HEIGHT / _FONTHEIGHT / 2)
    x = INT(_WIDTH / _FONTWIDTH / 2)

    tl = LEN(text$) + limitnum 'total length of letter spacing used
    x = x - INT(tl / 2) 'recompute x based on text length

    LINE (x * 8 - 24, y * 16 - 32)-((x * 8) + (tl * 8) + 8, y * 16 + 16), bg&, BF
    LINE (x * 8 - 24, y * 16 - 32)-((x * 8) + (tl * 8) + 8, y * 16 + 16), fg&, B
    COLOR fg&, bg&

    LOCATE y, x: PRINT text$;
    y = CSRLIN: x = POS(1)

    IF limitnum = 0 THEN
        IF delay = -1 THEN
            a$ = INPUT$(1)
        ELSE
            _DELAY delay
        END IF
        IBOX$ = ""
    ELSE

        entry$ = ""

        DO
            a$ = INPUT$(1)

            '=== Return exits with data
            IF a$ = CHR$(13) THEN IBOX$ = entry$: EXIT DO
            '=== Handle the backspace
            IF a$ = CHR$(27) THEN IBOX$ = "": EXIT DO
            '=== ESC cancels
            IF a$ = CHR$(8) THEN 'Backspace goes back a space
                IF LEN(entry$) > 0 THEN
                    entry$ = MID$(entry$, 1, LEN(entry$) - 1)
                END IF
            ELSE
                'add letter entered, if not over limitnum
                IF LEN(entry$) < limitnum THEN
                    entry$ = entry$ + a$
                END IF
            END IF

            LOCATE y, x: PRINT SPACE$(limitnum);
            LOCATE y, x: PRINT entry$;

        LOOP

    END IF

    '=== Restore the whole screen
    _MEMCOPY scr2, scr2.OFFSET, scr2.SIZE TO scr1, scr1.OFFSET
    _MEMFREE scr1: _MEMFREE scr2

    '=== restore original y,x
    LOCATE origy, origx

END FUNCTION


